{% for type_annotation in type_annotation.get_local_types() -%}
{% if type_annotation.is_typed_dict() -%}
    [:material-code-braces: {{ type_annotation.name }}]({{ package.service_name.get_md_doc_link("type_defs", type_annotation.name) }}{{ ')' -}}
{% elif type_annotation.is_literal() -%}
    [:material-code-brackets: {{ type_annotation.name }}]({{ package.service_name.get_md_doc_link("literals", type_annotation.name) }}{{ ')' -}}
{% elif hasattr(type_annotation, 'stringify') -%}
    [{{ type_annotation.name }}{{ '](#' }}{{ get_anchor_link(type_annotation.name) }}{{ ')' -}}
{% else -%}
    {{ '`' -}}{{ type_annotation.render() }}{{ '`' -}}
{% endif -%}
{{ ' ' -}}
{% endfor -%}
