## How to install

### VSCode extension

Add [AWS Boto3](https://marketplace.visualstudio.com/items?itemName=Boto3typed.boto3-ide)
extension to your VSCode and run `AWS boto3: Quick Start` command.

Click `Auto-discover services` and select services you use in the current project.

### From PyPI with pip

Install `{{ package.data.PYPI_NAME }}` to add type annotations for `{{ package.library_name }}` package.

```bash
# install type annotations only for {{ package.library_name }}
python -m pip install {{ package.data.PYPI_NAME }}

# install {{ package.library_name }} type annotations
# for ec2, s3, rds, lambda, sqs, dynamo and cloudformation
python -m pip install '{{ package.data.PYPI_NAME }}[essential]'

# or install annotations for services you use
python -m pip install '{{ package.data.PYPI_NAME }}[acm,apigateway]'

{% if package.data.PYPI_LITE_NAME %}
# Lite version does not provide session.client/resource overloads
# it is more RAM-friendly, but requires explicit type annotations
python -m pip install '{{ package.data.PYPI_LITE_NAME }}[essential]'
{% endif %}
```

### From conda-forge

Add `conda-forge` to your channels with:

```bash
conda config --add channels conda-forge
conda config --set channel_priority strict
```

Once the `conda-forge` channel has been enabled, `{{ package.data.PYPI_NAME }}`
and `{{ package.data.PYPI_NAME }}-essential` can be installed with:

```bash
conda install {{ package.data.PYPI_NAME }} {{ package.data.PYPI_NAME }}-essential
```

It is possible to list all of the versions of `{{ package.data.PYPI_NAME }}` available on your platform with:

```bash
conda search {{ package.data.PYPI_NAME }} --channel conda-forge
```

## How to uninstall

```bash
# uninstall {{ package.data.PYPI_NAME }}
python -m pip uninstall -y {{ package.data.PYPI_NAME }}
```
