# Type annotations for {{ package.library_name }}

Auto-generated documentation for [{{ package.library_name }}](https://github.com/aio-libs/aiobotocore) type annotations package [{{ package.pypi_name }}](https://pypi.org/project/{{ package.pypi_name }}/).

Generated by [mypy-boto3-builder]({{ builder_repo_url }}).

{% include "common/how_to_install.md.jinja2" with context %}

## Packages
- `types-aiobotocore` - [aiobotocore](https://github.com/aio-libs/aiobotocore) type annotations with `session.create_client` overloads
- `types-aiobotocore-lite` - [aiobotocore](https://github.com/aio-libs/aiobotocore) type annotations without `session.create_client` overloads
- `botocore-stubs` - [botocore](https://github.com/boto/botocore) type annotations
{% for service_name in package.service_names %}
- [{{ package.get_service_pypi_name(service_name) }}](./{{ package.get_module_name(service_name) }}/README.md) for [{{ service_name.class_name }}]({{ service_name.boto3_doc_link }}) service
{% endfor %}
