from collections.abc import Generator
from typing import Any, Iterable

from botocore.eventstream import EventStream
from requests.models import Response

class AioEventStream(EventStream):
    def __iter__(self) -> Iterable[Any]: ...
    def __aiter__(self) -> Iterable[Any]: ...
    async def __anext__(self) -> Generator[Any, None, None]: ...
    async def get_initial_response(self) -> Response: ...
