# mypy-boto3

[![PyPI - {{ package.pypi_name }}](https://img.shields.io/pypi/v/{{ package.pypi_name }}.svg?color=blue)](https://pypi.org/project/{{ package.pypi_name }})
[![PyPI - Python Version](https://img.shields.io/pypi/pyversions/{{ package.pypi_name }}.svg?color=blue)](https://pypi.org/project/{{ package.pypi_name }})
[![Docs](https://img.shields.io/readthedocs/mypy-boto3-builder.svg?color=blue)](https://mypy-boto3-builder.readthedocs.io/)
[![PyPI - Downloads](https://img.shields.io/pypi/dm/{{ package.pypi_name }}?color=blue)](https://pypistats.org/packages/{{ package.pypi_name }})

![boto3.typed]({{ builder_repo_url }}/raw/main/logo.png)

Dynamic [boto3 {{ package.library_version }}](https://boto3.amazonaws.com/v1/documentation/api/{{ package.library_version }}/index.html)
type annotations builder for [boto3-stubs](https://pypi.org/project/boto3-stubs/).

Generated by [mypy-boto3-builder {{ builder_version }}]({{ builder_repo_url }}).

More information can be found on [boto3-stubs](https://pypi.org/project/boto3-stubs/) page.

See how it helps to find and fix potential bugs:

![boto3-stubs demo]({{ builder_repo_url }}/raw/main/demo.gif)

## How to install

```bash
# Install type annotations for boto3 services you use
python -m pip install 'boto3-stubs[s3,ec2]'

# Lite version does not provide session.client/resource overloads
# it is more RAM-friendly, but requires explicit type annotations
python -m pip install 'boto3-stubs-lite[s3,ec2]'
```

## Usage

This tool is not longer required for annotations to work. Do not use it,
install [boto3-stubs](https://pypi.org/project/boto3-stubs/) instead.
