{% for import_record in package.get_init_required_import_records() -%}
    {% include "common/import_record_fallback.py.jinja2" with context %}
{% endfor -%}

__all__ = (
{% for name in package.get_all_names() -%}
    {{ '"' -}}
    {{ name -}}
    {{ '"' -}}
    {{ ",\n" if not loop.last or loop.first else "\n" }}
{% endfor -%}
)

__author__: str
__version__: str

DEFAULT_SESSION: Optional[Session]

def setup_default_session(
    aws_access_key_id: Optional[str] = ...,
    aws_secret_access_key: Optional[str] = ...,
    aws_session_token: Optional[str] = ...,
    region_name: Optional[str] = ...,
    botocore_session: Optional[str] = ...,
    profile_name: Optional[str] = ...,
) -> Session: ...
def set_stream_logger(
    name: str = ..., level: int = ..., format_string: Optional[str] = ...
) -> None: ...
def _get_default_session() -> Session: ...

class NullHandler(logging.Handler):
    def emit(self, record: Any) -> Any: ...

{% for function in package.init_functions -%}
    {% include "common/function.py.jinja2" with context -%}
    {{ '\n' -}}
{% endfor -%}