from typing import Any

async def inject_presigned_url_ec2(
    params: Any, request_signer: Any, model: Any, **kwargs: Any
) -> None: ...
async def inject_presigned_url_rds(
    params: Any, request_signer: Any, model: Any, **kwargs: Any
) -> None: ...
